# Deeper Dive: DIDcomm Messaging

DIDcomm peer-to-peer messages are asynchronous messages that one agent sends to another - for example, Faber would send to Alice. In between, there may be other agents and message processing, but at the edges, Faber appears to be messaging directly with Alice using encryption based on the DIDs and DIDDocs that the two shared when establishing a connection. The messages are JSON-LD-friendly messages with a "type" that defines the namespace, protocol, protocol version and type of the message, an "id" that is GUID for the message, and additional fields as required by the message type. The namespace is currently defined to be a public DID that should be globally resolvable to a protocol specification. Currently, "core" messages use a DID that is not yet globally resolvable - Daniel Hardman has the keys associated with the DID.

Link: [Message Types](https://github.com/hyperledger/aries-rfcs/blob/master/concepts/0020-message-types/README.md)

As protocols are executed, the data associated with the protocol is stored in the (currently named) wallet of the agent. The data primarily consists of the state object for that instance of the protocol, and any artifacts of running the protocol. For example, when establishing a connection, the metadata associated with the connection (DIDs, DID Documents and private keys) is stored in the agent's wallet. Likewise, ledger data is cached in the wallet (DIDs, schema, credential definitions, etc.) and credentials. This is taken care of by the Aries agent and the protocols configured into the agent.

## Message Decorators

In addition to protocol specific data elements in messages, messages can include "decorators", standardized message elements that define cross-cutting behavior. The most common example is the "thread" decorator, which is used to link the messages in a protocol instance. As messages go back and forth between agents to complete an instance of a protocol (e.g. issuing a credential), the [thread decorator](https://github.com/hyperledger/aries-rfcs/tree/master/concepts/0008-message-id-and-threading) data elements let the agents know to which protocol instance the message belongs. Other currently defined examples of decorators include [attachments](https://github.com/hyperledger/aries-rfcs/tree/master/concepts/0017-attachments), [localization](https://github.com/hyperledger/aries-rfcs/blob/master/features/0043-l10n/README.md), [tracing](https://github.com/hyperledger/aries-rfcs/blob/master/features/0034-message-tracing/README.md) and [timing](https://github.com/hyperledger/aries-rfcs/blob/master/features/0032-message-timing/README.md). Decorators are often processed by the core of the agent, but some are processed by the protocol message handlers. For example, the thread decorator processed to retrieve the protocol state object for that instance (thread) of the protocol before control is passed to the protocol message handler.
